@@ -28,17 +28,13 @@
 
 #include "libaudgui-gtk.h"
 
-#if GTK_CHECK_VERSION (3, 12, 0)
-#define gtk_widget_set_margin_left gtk_widget_set_margin_start
-#endif
-
 static void widget_changed (GtkWidget * widget, const PreferencesWidget * w)
 {
     switch (w->type)
     {
     case PreferencesWidget::CheckButton:
     {
-        gboolean set = gtk_toggle_button_get_active ((GtkToggleButton *) widget);
+        bool set = gtk_toggle_button_get_active ((GtkToggleButton *) widget);
         w->cfg.set_bool (set);
 
         auto child = (GtkWidget *) g_object_get_data ((GObject *) widget, "child");
@@ -49,10 +45,17 @@ static void widget_changed (GtkWidget * widget, const PreferencesWidget * w)
     }
 
     case PreferencesWidget::RadioButton:
-        if (gtk_toggle_button_get_active ((GtkToggleButton *) widget))
+    {
+        bool set = gtk_toggle_button_get_active ((GtkToggleButton *) widget);
+        if (set)
             w->cfg.set_int (w->data.radio_btn.value);
 
+        auto child = (GtkWidget *) g_object_get_data ((GObject *) widget, "child");
+        if (child)
+            gtk_widget_set_sensitive (child, set);
+
         break;
+    }
 
     case PreferencesWidget::SpinButton:
         if (w->cfg.type == WidgetConfig::Int)
@@ -70,6 +73,13 @@ static void widget_changed (GtkWidget * widget, const PreferencesWidget * w)
         w->cfg.set_string (gtk_entry_get_text ((GtkEntry *) widget));
         break;
 
+    case PreferencesWidget::FileEntry:
+    {
+        String uri = audgui_file_entry_get_uri (widget);
+        w->cfg.set_string (uri ? uri : "");
+        break;
+    }
+
     case PreferencesWidget::ComboBox:
     {
         auto items = (const ComboItem *) g_object_get_data ((GObject *) widget, "comboitems");
@@ -124,6 +134,10 @@ static void widget_update (void *, void * widget)
         gtk_entry_set_text ((GtkEntry *) widget, w->cfg.get_string ());
         break;
 
+    case PreferencesWidget::FileEntry:
+        audgui_file_entry_set_uri ((GtkWidget *) widget, w->cfg.get_string ());
+        break;
+
     case PreferencesWidget::ComboBox:
         combobox_update ((GtkWidget *) widget, w);
         break;
@@ -162,6 +176,7 @@ static void widget_init (GtkWidget * widget, const PreferencesWidget * w)
         break;
 
     case PreferencesWidget::Entry:
+    case PreferencesWidget::FileEntry:
     case PreferencesWidget::ComboBox:
         g_signal_connect (widget, "changed", (GCallback) widget_changed, (void *) w);
         break;
@@ -253,6 +268,33 @@ static void create_entry (const PreferencesWidget * widget, GtkWidget * * label,
     widget_init (* entry, widget);
 }
 
+/* WIDGET_FILE_ENTRY */
+
+static void create_file_entry (const PreferencesWidget * widget,
+ GtkWidget * * label, GtkWidget * * entry, const char * domain)
+{
+    switch (widget->data.file_entry.mode)
+    {
+    case FileSelectMode::File:
+        * entry = audgui_file_entry_new (GTK_FILE_CHOOSER_ACTION_OPEN, _("Choose File"));
+        break;
+
+    case FileSelectMode::Folder:
+        * entry = audgui_file_entry_new (GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, _("Choose Folder"));
+        break;
+    }
+
+    gtk_widget_set_hexpand (* entry, true);
+
+    if (widget->label)
+    {
+        * label = gtk_label_new (dgettext (domain, widget->label));
+        gtk_widget_set_halign (* label, GTK_ALIGN_END);
+    }
+
+    widget_init (* entry, widget);
+}
+
 /* WIDGET_COMBO_BOX */
 
 static void combobox_update (GtkWidget * combobox, const PreferencesWidget * widget)
@@ -342,6 +384,10 @@ static void fill_table (GtkWidget * table,
                 create_entry (& w, & widget_left, & widget_middle, domain);
                 break;
 
+            case PreferencesWidget::FileEntry:
+                create_file_entry (& w, & widget_left, & widget_middle, domain);
+                break;
+
             case PreferencesWidget::ComboBox:
                 create_cbox (& w, & widget_left, & widget_middle, domain);
                 break;
@@ -371,7 +417,7 @@ void audgui_create_widgets_with_domain (GtkWidget * box,
 {
     GtkWidget * widget = nullptr, * child_box = nullptr;
     bool disable_child = false;
-    GSList * radio_btn_group = nullptr;
+    GSList * radio_btn_group[2] = {};
 
     int indent = 0;
     int spacing = 0;
@@ -400,8 +446,11 @@ void audgui_create_widgets_with_domain (GtkWidget * box,
         widget = nullptr;
         disable_child = false;
 
-        if (radio_btn_group && w.type != PreferencesWidget::RadioButton)
-            radio_btn_group = nullptr;
+        if (w.type != PreferencesWidget::RadioButton)
+            radio_btn_group[w.child] = nullptr;
+
+        if (! w.child)
+            radio_btn_group[true] = nullptr;
 
         int pad_left = indent;
         int pad_top = spacing;
@@ -447,9 +496,10 @@ void audgui_create_widgets_with_domain (GtkWidget * box,
             }
 
             case PreferencesWidget::RadioButton:
-                widget = gtk_radio_button_new_with_mnemonic (radio_btn_group,
-                 dgettext (domain, w.label));
-                radio_btn_group = gtk_radio_button_get_group ((GtkRadioButton *) widget);
+                widget = gtk_radio_button_new_with_mnemonic
+                 (radio_btn_group[w.child], dgettext (domain, w.label));
+                radio_btn_group[w.child] = gtk_radio_button_get_group ((GtkRadioButton *) widget);
+                disable_child = (w.cfg.get_int () != w.data.radio_btn.value);
                 widget_init (widget, & w);
                 break;
 
@@ -501,11 +551,16 @@ void audgui_create_widgets_with_domain (GtkWidget * box,
                 break;
 
             case PreferencesWidget::Entry:
+            case PreferencesWidget::FileEntry:
             {
                 widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
                 GtkWidget * entry = nullptr;
-                create_entry (& w, & label, & entry, domain);
+
+                if (w.type == PreferencesWidget::FileEntry)
+                    create_file_entry (& w, & label, & entry, domain);
+                else
+                    create_entry (& w, & label, & entry, domain);
 
                 if (label)
                     gtk_box_pack_start ((GtkBox *) widget, label, false, false, 0);
