@@ -1,5 +1,5 @@
 /*
- * prefs-window.c
+ * prefs-window.cc
  * Copyright 2006-2014 William Pitcock, Tomasz Moń, Michael Färber, and
  *                     John Lindgren
  *
@@ -95,18 +95,19 @@ static const PluginCategory plugin_categories[] = {
 };
 
 static const TitleFieldTag title_field_tags[] = {
-    { N_("Artist")     , "${artist}" },
-    { N_("Album")      , "${album}" },
-    { N_("Title")      , "${title}" },
+    { N_("Artist")      , "${artist}" },
+    { N_("Album")       , "${album}" },
+    { N_("Album artist"), "${album-artist}" },
+    { N_("Title")       , "${title}" },
     { N_("Track number"), "${track-number}" },
-    { N_("Genre")      , "${genre}" },
+    { N_("Genre")       , "${genre}" },
     { N_("File name")   , "${file-name}" },
     { N_("File path")   , "${file-path}" },
-    { N_("Date")       , "${date}" },
-    { N_("Year")       , "${year}" },
-    { N_("Comment")    , "${comment}" },
-    { N_("Codec")      , "${codec}" },
-    { N_("Quality")    , "${quality}" }
+    { N_("Date")        , "${date}" },
+    { N_("Year")        , "${year}" },
+    { N_("Comment")     , "${comment}" },
+    { N_("Codec")       , "${codec}" },
+    { N_("Quality")     , "${quality}" }
 };
 
 static const ComboItem chardet_detector_presets[] = {
@@ -125,12 +126,26 @@ static const ComboItem chardet_detector_presets[] = {
 };
 
 static const ComboItem bitdepth_elements[] = {
+    ComboItem (N_("Automatic"), -1),
     ComboItem ("16", 16),
     ComboItem ("24", 24),
     ComboItem ("32", 32),
     ComboItem (N_("Floating point"), 0)
 };
 
+static const ComboItem record_elements[] = {
+    ComboItem (N_("As decoded"), (int) OutputStream::AsDecoded),
+    ComboItem (N_("After applying ReplayGain"), (int) OutputStream::AfterReplayGain),
+    ComboItem (N_("After applying effects"), (int) OutputStream::AfterEffects),
+    ComboItem (N_("After applying equalization"), (int) OutputStream::AfterEqualizer)
+};
+
+static const ComboItem replaygainmode_elements[] = {
+    ComboItem (N_("Track"), (int) ReplayGainMode::Track),
+    ComboItem (N_("Album"), (int) ReplayGainMode::Album),
+    ComboItem (N_("Based on shuffle"), (int) ReplayGainMode::Automatic)
+};
+
 static Index<ComboItem> iface_combo_elements;
 static int iface_combo_selected;
 static GtkWidget * iface_prefs_box;
@@ -140,10 +155,10 @@ static void iface_combo_changed ();
 static void * iface_create_prefs_box ();
 
 static const PreferencesWidget appearance_page_widgets[] = {
-    WidgetLabel (N_("<b>Interface Settings</b>")),
-    WidgetCombo (N_("Interface plugin:"),
+    WidgetCombo (N_("Interface:"),
         WidgetInt (iface_combo_selected, iface_combo_changed),
         {0, iface_combo_fill}),
+    WidgetSeparator ({true}),
     WidgetCustomGTK (iface_create_prefs_box)
 };
 
@@ -160,7 +175,7 @@ static void output_bit_depth_changed ();
 
 static const PreferencesWidget output_combo_widgets[] = {
     WidgetCombo (N_("Output plugin:"),
-        WidgetInt (output_combo_selected, output_combo_changed),
+        WidgetInt (output_combo_selected, output_combo_changed, "audgui update output combo"),
         {0, output_combo_fill}),
     WidgetCustomGTK (output_create_config_button),
     WidgetCustomGTK (output_create_about_button)
@@ -182,12 +197,10 @@ static const PreferencesWidget record_buttons[] = {
 static const PreferencesWidget gain_table[] = {
     WidgetSpin (N_("Amplify all files:"),
         WidgetFloat (0, "replay_gain_preamp"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD),
+        {-15, 15, 0.1, N_("dB")}),
     WidgetSpin (N_("Amplify untagged files:"),
         WidgetFloat (0, "default_gain"),
-        {-15, 15, 0.1, N_("dB")},
-        WIDGET_CHILD)
+        {-15, 15, 0.1, N_("dB")})
 };
 
 static const PreferencesWidget audio_page_widgets[] = {
@@ -199,24 +212,27 @@ static const PreferencesWidget audio_page_widgets[] = {
     WidgetSpin (N_("Buffer size:"),
         WidgetInt (0, "output_buffer_size"),
         {100, 10000, 1000, N_("ms")}),
-    WidgetCustomGTK (record_create_checkbox),
-    WidgetBox ({{record_buttons}, true},
-        WIDGET_CHILD),
     WidgetCheck (N_("Soft clipping"),
         WidgetBool (0, "soft_clipping")),
     WidgetCheck (N_("Use software volume control (not recommended)"),
         WidgetBool (0, "software_volume_control")),
-    WidgetLabel (N_("<b>Replay Gain</b>")),
-    WidgetCheck (N_("Enable Replay Gain"),
+    WidgetLabel (N_("<b>Recording Settings</b>")),
+    WidgetCustomGTK (record_create_checkbox),
+    WidgetBox ({{record_buttons}, true},
+        WIDGET_CHILD),
+    WidgetCombo (N_("Record stream:"),
+        WidgetInt (0, "record_stream"),
+        {{record_elements}}),
+    WidgetLabel (N_("<b>ReplayGain</b>")),
+    WidgetCheck (N_("Enable ReplayGain"),
         WidgetBool (0, "enable_replay_gain")),
-    WidgetCheck (N_("Album mode"),
-        WidgetBool (0, "replay_gain_album"),
+    WidgetCombo (N_("Mode:"),
+        WidgetInt (0, "replay_gain_mode"),
+        {{replaygainmode_elements}},
         WIDGET_CHILD),
     WidgetCheck (N_("Prevent clipping (recommended)"),
         WidgetBool (0, "enable_clipping_prevention"),
         WIDGET_CHILD),
-    WidgetLabel (N_("<b>Adjust Levels</b>"),
-        WIDGET_CHILD),
     WidgetTable ({{gain_table}},
         WIDGET_CHILD)
 };
@@ -279,13 +295,15 @@ static const PreferencesWidget playlist_page_widgets[] = {
     WidgetLabel (N_("<b>Song Display</b>")),
     WidgetCheck (N_("Show song numbers"),
         WidgetBool (0, "show_numbers_in_pl", send_title_change)),
-    WidgetCheck (N_("Show leading zeroes (02:00 instead of 2:00)"),
+    WidgetCheck (N_("Show leading zeroes (02:00 vs. 2:00)"),
         WidgetBool (0, "leading_zero", send_title_change)),
+    WidgetCheck (N_("Show hours separately (1:30:00 vs. 90:00)"),
+        WidgetBool (0, "show_hours", send_title_change)),
     WidgetCustomGTK (create_titlestring_table),
     WidgetLabel (N_("<b>Compatibility</b>")),
     WidgetCheck (N_("Interpret \\ (backward slash) as a folder delimiter"),
         WidgetBool (0, "convert_backslash")),
-    WidgetTable ({{chardet_elements}}),
+    WidgetTable ({{chardet_elements}})
 };
 
 static const PreferencesWidget song_info_page_widgets[] = {
@@ -465,13 +483,10 @@ static void fill_category_list (GtkTreeView * treeview, GtkNotebook * notebook)
          gettext (category.name), -1);
 
         StringBuf path = filename_build ({data_dir, "images", category.icon_path});
-        GdkPixbuf * img = gdk_pixbuf_new_from_file (path, nullptr);
+        AudguiPixbuf img (gdk_pixbuf_new_from_file (path, nullptr));
 
         if (img)
-        {
-            gtk_list_store_set (store, & iter, CATEGORY_VIEW_COL_ICON, img, -1);
-            g_object_unref (img);
-        }
+            gtk_list_store_set (store, & iter, CATEGORY_VIEW_COL_ICON, img.get (), -1);
     }
 
     g_object_unref (store);
@@ -539,7 +554,6 @@ static void * create_titlestring_table ()
 
     GtkWidget * titlestring_help_button = gtk_button_new ();
     gtk_widget_set_can_focus (titlestring_help_button, false);
-    gtk_button_set_focus_on_click ((GtkButton *) titlestring_help_button, false);
     gtk_button_set_relief ((GtkButton *) titlestring_help_button, GTK_RELIEF_HALF);
     gtk_grid_attach ((GtkGrid *) grid, titlestring_help_button, 2, 1, 1, 1);
 
@@ -620,13 +634,20 @@ static void create_appearance_category ()
 
 static void output_combo_changed ()
 {
-    PluginHandle * plugin = aud_plugin_list (PluginType::Output)[output_combo_selected];
+    auto & list = aud_plugin_list (PluginType::Output);
+    PluginHandle * plugin = list[output_combo_selected];
 
     if (aud_plugin_enable (plugin, true))
     {
         gtk_widget_set_sensitive (output_config_button, aud_plugin_has_configure (plugin));
         gtk_widget_set_sensitive (output_about_button, aud_plugin_has_about (plugin));
     }
+    else
+    {
+        /* set combo box back to current output */
+        output_combo_selected = list.find (aud_plugin_get_current (PluginType::Output));
+        hook_call ("audgui update output combo", nullptr);
+    }
 }
 
 static ArrayRef<ComboItem> output_combo_fill ()
@@ -719,7 +740,7 @@ static void record_update (void * = nullptr, void * = nullptr)
 
         gtk_widget_set_sensitive (record_checkbox, true);
         gtk_button_set_label ((GtkButton *) record_checkbox,
-         str_printf (_("Record audio stream using %s"), aud_plugin_get_name (p)));
+         str_printf (_("Enable audio stream recording with %s"), aud_plugin_get_name (p)));
         gtk_toggle_button_set_active ((GtkToggleButton *) record_checkbox, enabled);
         gtk_widget_set_sensitive (record_config_button, enabled && aud_plugin_has_configure (p));
         gtk_widget_set_sensitive (record_about_button, enabled && aud_plugin_has_about (p));
